﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;

namespace DotP2013_Card_Builder
{
	class CardInfo
	{
		public enum CardRarity
		{
			Invalid = -1,
			Land = 0,
			Common,
			Uncommon,
			Rare,
			Mythic,
			Special,
			Promo
		}

		public SortedList<string, KeyValuePair<string, int>> m_colCardNames;
		public CardType m_ctCardType;
		public SortedList<string, string> m_colFlavourText;
		public List<Ability> m_lstAbilities;
		public String m_strCastingCost;
		public String m_strArtist;
		public String m_strRarity;
		public String m_strExpansion;
		public String m_strPower;
		public String m_strToughness;

		public CardInfo()
		{
			m_colCardNames = null;
			m_ctCardType = null;
			m_colFlavourText = null;
			m_lstAbilities = null;
			m_strCastingCost = "";
			m_strArtist = "";
			m_strRarity = "";
			m_strExpansion = "";
			m_strPower = "";
			m_strToughness = "";
		}

		public int MultiverseId
		{
			get
			{
				if (m_colCardNames != null)
				{
					return m_colCardNames["English"].Value;
				}
				else
					return 0;
			}
		}

		public String GetCardName(string strLang)
		{
			if (m_colCardNames != null)
			{
				if (m_colCardNames.IndexOfKey(strLang) > -1)
					return m_colCardNames[strLang].Key;
				else
					return m_colCardNames["English"].Key;
			}
			else
				return null;
		}

		public CardType CardType
		{
			get
			{
				return m_ctCardType;
			}
		}

		public string GetFlavourText(string strLang)
		{
			if (m_colFlavourText != null)
			{
				if (m_colFlavourText.IndexOfKey(strLang) > -1)
					return m_colFlavourText[strLang];
				else
					return m_colFlavourText["English"];
			}
			else
				return null;
		}

		public bool HasFlavourText()
		{
			if (m_colFlavourText != null)
				return m_colFlavourText.Count > 0;
			else
				return false;
		}

		public List<Ability> Abilities
		{
			get
			{
				return m_lstAbilities;
			}
		}

		public String CastingCost
		{
			get
			{
				return m_strCastingCost;
			}
		}

		public String Power
		{
			get
			{
				return m_strPower;
			}
		}

		public String Toughness
		{
			get
			{
				return m_strToughness;
			}
		}

		public String Artist
		{
			get
			{
				return m_strArtist;
			}
		}

		public String Rarity
		{
			get
			{
				return m_strRarity;
			}
		}

		public CardRarity RarityEnum
		{
			get
			{
				if (m_strRarity != null)
				{
					CardRarity cr = CardRarity.Invalid;
					if (m_strRarity.Equals("Land", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Land;
					else if (m_strRarity.Equals("Common", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Common;
					else if (m_strRarity.Equals("Uncommon", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Uncommon;
					else if (m_strRarity.Equals("Rare", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Rare;
					else if (m_strRarity.Equals("Mythic Rare", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Mythic;
					else if (m_strRarity.Equals("Special", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Special;
					else if (m_strRarity.Equals("Promo", StringComparison.OrdinalIgnoreCase))
						cr = CardRarity.Promo;
					return cr;
				}
				else
					return CardRarity.Invalid;
			}
		}

		public String Expansion
		{
			get
			{
				return m_strExpansion;
			}
		}

		protected virtual Ability GetNewAbility()
		{
			return new Ability();
		}

		protected virtual CardType GetNewCardType(string strType)
		{
			return new CardType(strType);
		}

		public bool GetInformation(int nMultiverseId)
		{
			bool bReturn = false;
			m_colCardNames = Utils.GetCardLanguages(nMultiverseId);
			if (m_colCardNames.IndexOfKey("English") > -1)
			{
				GetBasicCardInformation(m_colCardNames["English"].Value);
				foreach (string strLang in m_colCardNames.Keys)
				{
					if (!strLang.Equals("English", StringComparison.OrdinalIgnoreCase))
						GetLocalizedCardInformation(strLang, m_colCardNames[strLang].Value);
				}
				bReturn = true;
			}
			return bReturn;
		}

		/*public String GetInformation(string strName)
		{
			String strReturn = "";
			String strSource = Utils.GetGathererSearchPage(strName);
			XDocument xdSource = XDocument.Parse(strSource);
			XElement xeCardTitle = Utils.FindXElementWithId(xdSource, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_ctl00_listRepeater_ctl00_cardTitle");
			XElement xeCardName = Utils.FindXElementWithId(xdSource, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_nameRow");
			if (xeCardTitle != null)
			{
				// We have multiple results in a search list.
			}
			else if (xeCardName != null)
			{
				// We have a single result page same as if we started with a multiverse id, but we don't have the id yet.
			}
			else
			{
				// We have no results on the card.
			}
			strReturn = strSource;
			return strReturn;
		}//*/

		private void GetBasicCardInformation(int nEnglishId)
		{
			// We get the Oracle page for Card Type information.
			XDocument xdOraclePage = XDocument.Parse(Utils.GetGathererOracleCardPage(nEnglishId));
			// And the Printed page for all other English card information.
			XDocument xdPrintedPage = XDocument.Parse(Utils.GetGathererPrintedCardPage(nEnglishId));
			// By looking for the "Right Side" first I can reduce how many elements I have to look through for the rest of the searches.
			XElement xeCardInfoBlock = Utils.FindXElementWithId(xdPrintedPage, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_rightCol");
			XElement xeCastingCostRow = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_manaRow");
			if (xeCastingCostRow != null)
			{
				XElement xeCost = Utils.FindXElementWithClass(xeCastingCostRow, "value");
				if (xeCost != null)
				{
					string strCost = "";
					foreach (XElement xe in xeCost.Elements())
					{
						XAttribute xa = xe.Attribute("alt");
						if (xa != null)
							strCost += Utils.ConvertManaNameToSymbol(xa.Value.Trim());
					}
					m_strCastingCost = strCost;
				}
			}
			// This type row is the only place where we prefer to get information from the Oracle page.
			XElement xeCardTypeRow = Utils.FindXElementWithId(xdOraclePage, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_typeRow");
			if (xeCardTypeRow != null)
			{
				XElement xeCardType = Utils.FindXElementWithClass(xeCardTypeRow, "value");
				if (xeCardType != null)
					m_ctCardType = GetNewCardType(xeCardType.Value.Trim());
			}
			XElement xePowerToughness = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_ptRow");
			if (xePowerToughness != null)
			{
				XElement xeValue = Utils.FindXElementWithClass(xePowerToughness, "value");
				if (xeValue != null)
				{
					// Values are in the form "X / Y"
					string[] astrValues = xeValue.Value.Trim().Split('/');
					m_strPower = astrValues[0].Trim();
					m_strToughness = astrValues[1].Trim();
				}
			}
			XElement xeArtistCredit = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_ArtistCredit");
			if (xeArtistCredit != null)
			{
				m_strArtist = xeArtistCredit.Element("{" + xeArtistCredit.Name.NamespaceName + "}a").Value.Trim();
			}
			XElement xeRarityRow = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_rarityRow");
			if (xeRarityRow != null)
			{
				XElement xeValue = Utils.FindXElementWithClass(xeRarityRow, "value");
				if (xeValue != null)
				{
					// More often than not there is a <span> tag that actually contains the data, but I should also check for without.
					if (xeValue.HasElements)
					{
						XElement xeActualValue = xeValue.Element("{" + xeValue.Name.NamespaceName + "}span");
						if (xeActualValue != null)
							m_strRarity = xeActualValue.Value.Trim();
					}
					else
						m_strRarity = xeValue.Value.Trim();
				}
			}
			XElement xeExpansionSet = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_currentSetSymbol");
			if (xeExpansionSet != null)
			{
				m_strExpansion = xeExpansionSet.Value.Trim();
			}
			m_lstAbilities = new List<Ability>();
			XElement xeAbilitiesRow = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_textRow");
			if (xeAbilitiesRow != null)
			{
				XElement xeAbilitiesBlock = Utils.FindXElementWithClass(xeAbilitiesRow, "value");
				if (xeAbilitiesBlock != null)
				{
					// Each ability is in its' own div with the exception of multiple simple abilities which Wizards will sometimes put on a single line.
					foreach (XElement xeAbility in xeAbilitiesBlock.Elements())
					{
						string strAbility = Utils.ParseTextElement(xeAbility);
						string strAbilityCompare = strAbility;
						// This is not perfect because it will eliminate the entire ability text for things like Landfall/Threshold abilities.
						//	Which is why we don't necessarily use this for saving ability text.
						if (strAbility.IndexOf("|", StringComparison.OrdinalIgnoreCase) > -1)
						{
							strAbilityCompare = strAbility.Substring(0, strAbility.IndexOf("|", StringComparison.OrdinalIgnoreCase)).Trim();
						}
						bool bCommaPresent = (strAbilityCompare.IndexOf(",", StringComparison.OrdinalIgnoreCase) > -1) && (strAbilityCompare.IndexOf("(", StringComparison.OrdinalIgnoreCase) < 0);
						bCommaPresent |= (strAbilityCompare.IndexOf(",", StringComparison.OrdinalIgnoreCase) > -1) && (strAbilityCompare.IndexOf("(", StringComparison.OrdinalIgnoreCase) > -1)
							&& (strAbilityCompare.IndexOf(",", StringComparison.OrdinalIgnoreCase) < strAbilityCompare.IndexOf("(", StringComparison.OrdinalIgnoreCase));
						if (bCommaPresent)
						{
							string[] astrSplit = strAbilityCompare.Split(',');
							if (Ability.TextRepresentsSimpleAbility(astrSplit[0].Trim()))
							{
								// We actually have a comma delimited list of simple abilities which we need to mark
								//	so we can properly parse them for localized text.
								int nNumOnLine = 0;
								foreach (string strSimple in astrSplit)
								{
									nNumOnLine++;
									Ability abNew = GetNewAbility();
									if (Ability.TextRepresentsSimpleAbility(strSimple.Trim()))
										abNew.SimpleAbility = true;
									if (nNumOnLine > 1)
										abNew.CommaSpace = true;
									abNew.AbilityTextList.Add("English", Utils.GetPrettyQuotes(strSimple.Trim()));
									m_lstAbilities.Add(abNew);
								}
							}
							else
							{
								Ability abNew = GetNewAbility();
								abNew.AbilityTextList.Add("English", Utils.GetPrettyQuotes(strAbility.Trim()));
								m_lstAbilities.Add(abNew);
							}
						}
						else
						{
							Ability abNew = GetNewAbility();
							if (Ability.TextRepresentsSimpleAbility(strAbilityCompare.Trim()))
								abNew.SimpleAbility = true;
							abNew.AbilityTextList.Add("English", Utils.GetPrettyQuotes(strAbility.Trim()));
							m_lstAbilities.Add(abNew);
						}
					}
				}
			}
			m_colFlavourText = new SortedList<string, string>();
			XElement xeFlavourText = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_FlavorText");
			if (xeFlavourText != null)
			{
				string strText = "";
				// The flavour text is put into multiple divs with italics as well (though incorrectly done for multiline flavours).
				foreach (XElement xeText in xeFlavourText.Elements())
				{
					strText += xeText.Value.Trim().Replace(((char)8232).ToString(), "\r\n") + "\r\n";
				}
				m_colFlavourText.Add("English", Utils.GetPrettyQuotes(strText.Trim()));
			}
		}

		private void GetLocalizedCardInformation(string strLang, int nMultiverseId)
		{
			XDocument xdPrintedPage = XDocument.Parse(Utils.GetGathererPrintedCardPage(nMultiverseId));
			XElement xeCardInfoBlock = Utils.FindXElementWithId(xdPrintedPage, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_rightCol");
			XElement xeAbilitiesRow = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_textRow");
			if (xeAbilitiesRow != null)
			{
				XElement xeAbilitiesBlock = Utils.FindXElementWithClass(xeAbilitiesRow, "value");
				if (xeAbilitiesBlock != null)
				{
					int nAbilityIndex = 0;
					// Each ability is in its' own div with the exception of multiple simple abilities which Wizards will sometimes put on a single line.
					foreach (XElement xeAbility in xeAbilitiesBlock.Elements())
					{
						string strAbility = Utils.ParseTextElement(xeAbility);
						string strAbilityCompare = strAbility;
						// This is not perfect because it will eliminate the entire ability text for things like Landfall/Threshold abilities.
						//	Which is why we don't necessarily use this for saving ability text.
						if (strAbility.IndexOf("|", StringComparison.OrdinalIgnoreCase) > -1)
						{
							strAbilityCompare = strAbility.Substring(0, strAbility.IndexOf("|", StringComparison.OrdinalIgnoreCase)).Trim();
						}
						if (nAbilityIndex < m_lstAbilities.Count)
						{
							Ability abTemp = m_lstAbilities[nAbilityIndex];
							bool bCommaPresent = (strAbilityCompare.IndexOf(",", StringComparison.OrdinalIgnoreCase) > -1) && (strAbilityCompare.IndexOf("(", StringComparison.OrdinalIgnoreCase) < 0);
							bCommaPresent |= (strAbilityCompare.IndexOf(",", StringComparison.OrdinalIgnoreCase) > -1) && (strAbilityCompare.IndexOf("(", StringComparison.OrdinalIgnoreCase) > -1)
								&& (strAbilityCompare.IndexOf(",", StringComparison.OrdinalIgnoreCase) < strAbilityCompare.IndexOf("(", StringComparison.OrdinalIgnoreCase));
							// Compatibility with Japanese printings
							bCommaPresent |= (strAbilityCompare.IndexOf("、", StringComparison.OrdinalIgnoreCase) > -1) && (strAbilityCompare.IndexOf("（", StringComparison.OrdinalIgnoreCase) < 0);
							bCommaPresent |= (strAbilityCompare.IndexOf("、", StringComparison.OrdinalIgnoreCase) > -1) && (strAbilityCompare.IndexOf("（", StringComparison.OrdinalIgnoreCase) > -1)
								&& (strAbilityCompare.IndexOf("、", StringComparison.OrdinalIgnoreCase) < strAbilityCompare.IndexOf("（", StringComparison.OrdinalIgnoreCase));
							if (bCommaPresent)
							{
								if (abTemp.SimpleAbility)
								{
									string[] astrSplit = strAbilityCompare.Split(',', '、');
									foreach (string strSimple in astrSplit)
									{
										if (nAbilityIndex >= m_lstAbilities.Count)
											break;
										// May seem redundant, but it ensures we are always on the right ability.
										abTemp = m_lstAbilities[nAbilityIndex];
										abTemp.AbilityTextList.Add(strLang, Utils.GetPrettyQuotes(strSimple.Trim()));
										nAbilityIndex++;
									}
								}
								else
								{
									abTemp.AbilityTextList.Add(strLang, Utils.GetPrettyQuotes(strAbility.Trim()));
									nAbilityIndex++;
								}
							}
							else
							{
								abTemp.AbilityTextList.Add(strLang, Utils.GetPrettyQuotes(strAbility.Trim()));
								nAbilityIndex++;
							}
						}
					}
				}
			}
			XElement xeFlavourText = Utils.FindXElementWithId(xeCardInfoBlock, "ctl00_ctl00_ctl00_MainContent_SubContent_SubContent_FlavorText");
			if (xeFlavourText != null)
			{
				string strText = "";
				// The flavour text is put into multiple divs with italics as well (though incorrectly done for multi-line flavours).
				foreach (XElement xeText in xeFlavourText.Elements())
				{
					strText += xeText.Value.Trim().Replace(((char)8232).ToString(), "\r\n") + "\r\n";
				}
				m_colFlavourText.Add(strLang, Utils.GetPrettyQuotes(strText.Trim()));
			}
		}

		public virtual string GetCardCode()
		{
			throw new NotImplementedException();
		}
	}
}
