﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DotP2013_Card_Builder
{
	class CardType
	{
		private SortedSet<string> m_colSupertypes;
		private SortedSet<string> m_colTypes;
		private SortedSet<SubType> m_colSubTypes;

		public CardType(string strType)
		{
			strType = strType.Trim();
			m_colSupertypes = new SortedSet<string>();
			m_colTypes = new SortedSet<string>();
			m_colSubTypes = new SortedSet<SubType>();
			if ((strType.IndexOf('-') > -1) || (strType.IndexOf('—') > -1))
			{
				// Possible Supertypes, definite Type(s) and SubTypes.
				string[] astrTypes = strType.Split(new char[] { '-', '—' });
				string[] astrCardTypes = astrTypes[0].Trim().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
				string[] astrSubTypes = astrTypes[1].Trim().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
				for (int i = 0; i < astrCardTypes.Count(); i++)
				{
					if ((astrCardTypes[i].Equals("Basic", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("Legendary", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("World", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("Snow", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("Ongoing", StringComparison.OrdinalIgnoreCase)))
						m_colSupertypes.Add(astrCardTypes[i]);
					else
						m_colTypes.Add(astrCardTypes[i]);
				}
				for (int i = 0; i < astrSubTypes.Count(); i++)
				{
					m_colSubTypes.Add(GetNewSubType(astrSubTypes[i], i));
				}
			}
			else
			{
				// Possible Supertypes, definite Type(s) and No SubTypes.
				string[] astrCardTypes = strType.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
				for (int i = 0; i < astrCardTypes.Count(); i++)
				{
					if ((astrCardTypes[i].Equals("Basic", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("Legendary", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("World", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("Snow", StringComparison.OrdinalIgnoreCase)) ||
						(astrCardTypes[i].Equals("Ongoing", StringComparison.OrdinalIgnoreCase)))
						m_colSupertypes.Add(astrCardTypes[i]);
					else
						m_colTypes.Add(astrCardTypes[i]);
				}
			}
		}

		public SortedSet<string> Supertypes
		{
			get
			{
				return m_colSupertypes;
			}
		}

		public SortedSet<string> CardTypes
		{
			get
			{
				return m_colTypes;
			}
		}

		public SortedSet<SubType> SubTypes
		{
			get
			{
				return m_colSubTypes;
			}
		}

		protected virtual SubType GetNewSubType(string strName, int nOrder)
		{
			return new SubType(strName, nOrder);
		}

		public override string ToString()
		{
			string strReturn = "";
			foreach (string strType in m_colSupertypes)
				strReturn += strType + ' ';
			foreach (string strType in m_colTypes)
				strReturn += strType + ' ';
			if (m_colSubTypes.Count() > 0)
			{
				strReturn += "— ";
				foreach (SubType strType in m_colSubTypes)
					strReturn += strType.ToString() + ' ';
			}
			return strReturn.Trim();
		}

		public virtual string GetCardText()
		{
			throw new NotImplementedException();
		}
	}
}
