﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace DotP2013_Card_Builder
{
	class DotP2013Ability : Ability
	{
		public DotP2013Ability()
			: base()
		{ }

		public DotP2013Ability(string strAbility)
			: base(strAbility)
		{
			///TODO: Need to parse out Ability Text if available.
		}

		public override string GetCardAbilityCode()
		{
			if (this.StoredAbilityCode != null)
			{
				return this.StoredAbilityCode;
			}
			else
			{
				string strPadding = Globals.Padding;
				string strReturn = LookForTemplate(GetAbilityText("English"), strPadding);
				if (strReturn == null)
				{
					if (SimpleAbility)
					{
						// We need to switch out to the appropriate function to build this simple ability.
						strReturn = GetSimpleAbilityCode(GetAbilityText("English"), strPadding);
					}
					else
					{
						// This is not a simple ability or at least I wasn't able to detect it as one,
						//	so we'll write out a base Static Ability block with nothing else.
						strReturn = strPadding + "<STATIC_ABILITY" + AddCommaSpace() + ">\r\n";
						strReturn += GetLocalizedAbilityText();
						strReturn += strPadding + "</STATIC_ABILITY>\r\n";
					}
				}
				return strReturn;
			}
		}

		private string LookForTemplate(string strAbility, string strPadding)
		{
			string strReturn = DotP2013Templates.GetAbilityTemplate(strAbility);
			if (strReturn != null)
			{
				strReturn = ProcessTemplateForVariables(strReturn, strAbility, strPadding);
			}
			return strReturn;
		}

		private string ProcessTemplateForVariables(string strTemplate, string strFullAbilityText, string strPadding)
		{
			if (strTemplate.IndexOf("{$Value$}", StringComparison.OrdinalIgnoreCase) > -1)
			{
				// Look for additional value in ability text
				if (strFullAbilityText.IndexOf(" ") > -1)
				{
					int nStart = strFullAbilityText.IndexOf(" ") + 1;
					int nEnd = strFullAbilityText.IndexOf(" ", nStart);
					string strValue = "0";
					if (nEnd > -1)
						strValue = strFullAbilityText.Substring(nStart, nEnd - nStart);
					else
						strValue = strFullAbilityText.Substring(nStart);

					strTemplate = Regex.Replace(strTemplate, Regex.Escape("{$Value$}"), strValue, RegexOptions.IgnoreCase);
				}
				else
					strTemplate = Regex.Replace(strTemplate, Regex.Escape("{$Value$}"), "0", RegexOptions.IgnoreCase);
			}

			// See if the template has requested Localized text.
			if (strTemplate.IndexOf("{$LocalisedText$}", StringComparison.OrdinalIgnoreCase) > -1)
			{
				// Get the ability's localized text.
				strTemplate = Regex.Replace(strTemplate, Regex.Escape("{$LocalisedText$}"), GetLocalizedAbilityText().Trim(), RegexOptions.IgnoreCase);
			}

			// Check if the ability has support for CommaSpace.
			if (strTemplate.IndexOf("{$CommaSpace$}", StringComparison.OrdinalIgnoreCase) > -1)
			{
				strTemplate = Regex.Replace(strTemplate, Regex.Escape("{$CommaSpace$}"), (CommaSpace ? 1 : 0).ToString(), RegexOptions.IgnoreCase);
			}
			return strTemplate;
		}

		private string AddCommaSpace()
		{
			if (CommaSpace)
				return " commaspace=\"1\"";
			else
				return "";
		}

		private string GetLocalizedAbilityText()
		{
			string strPadding = Globals.Padding;
			string strDblPadding = strPadding + strPadding;
			string strReturn;
			strReturn = strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("English") + "\"><![CDATA[" + GetAbilityText("English") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("French") + "\"><![CDATA[" + GetAbilityText("French") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Spanish") + "\"><![CDATA[" + GetAbilityText("Spanish") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("German") + "\"><![CDATA[" + GetAbilityText("German") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Italian") + "\"><![CDATA[" + GetAbilityText("Italian") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Japanese") + "\"><![CDATA[" + GetAbilityText("Japanese") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Korean") + "\"><![CDATA[" + GetAbilityText("Korean") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Russian") + "\"><![CDATA[" + GetAbilityText("Russian") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Portuguese") + "\"><![CDATA[" + GetAbilityText("Portuguese") + "]]></LOCALISED_TEXT>\r\n";
			return strReturn;
		}

		private string GetSimpleAbilityCode(string strText, string strPadding)
		{
			string strReturn = "";
			if (strText.StartsWith("Annihilator", StringComparison.OrdinalIgnoreCase))
			{
				int nNum = 0;
				if (strText.IndexOf(" ", 12) > -1)
					nNum = Int32.Parse(strText.Substring(12, strText.IndexOf(" ", 12, StringComparison.OrdinalIgnoreCase) - 12));
				else
					nNum = Int32.Parse(strText.Substring(12));
				if (nNum > 0)
					strReturn = GetAnnihilatorAbilityCode(nNum);
			}
			else if (strText.StartsWith("Battle Cry", StringComparison.OrdinalIgnoreCase))
				strReturn = GetBattleCryAbilityCode();
			else if (strText.StartsWith("Double strike", StringComparison.OrdinalIgnoreCase))
				strReturn = GetSimpleCharacteristicAbilityCode("DoubleStrike");
			else if (strText.StartsWith("Equip ", StringComparison.OrdinalIgnoreCase))
			{
				int nNum = Int32.Parse(strText.Substring(7, strText.IndexOf("}", 7, StringComparison.OrdinalIgnoreCase) - 7));
				if (nNum > 0)
					strReturn = GetEquipAbilityCode(nNum);
			}
			else if (strText.StartsWith("Exalted", StringComparison.OrdinalIgnoreCase))
				strReturn = GetExaltedAbilityCode();
			else if (strText.StartsWith("First strike", StringComparison.OrdinalIgnoreCase))
				strReturn = GetSimpleCharacteristicAbilityCode("FirstStrike");
			else if (strText.StartsWith("Flanking", StringComparison.OrdinalIgnoreCase))
				strReturn = GetFlankingAbilityCode();
			else if (strText.Equals("Flash", StringComparison.OrdinalIgnoreCase))
				strReturn = GetSimpleCharacteristicAbilityCodeInHand("Flash");
			else
			{
				// Everything else just needs to make sure it is in Proper Case.
				string strProper;
				if (strText.IndexOf(' ') > -1)
					strProper = strText.Substring(0, 1).ToUpper() + strText.Substring(1, strText.IndexOf(' ') - 1).ToLower();
				else
					strProper = strText.Substring(0, 1).ToUpper() + strText.Substring(1).ToLower();
				strReturn = GetSimpleCharacteristicAbilityCode(strProper);
			}
			return strReturn;
		}

		private string GetSimpleCharacteristicAbilityCode(string strText)
		{
			string strPadding = Globals.Padding;
			string strReturn = Utils.PaddingForLevel(strPadding, 1) + "<STATIC_ABILITY" + AddCommaSpace() + ">\r\n";
			strReturn += GetLocalizedAbilityText();
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<CONTINUOUS_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "RSN_" + strText + "( Object() )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</CONTINUOUS_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 1) + "</STATIC_ABILITY>\r\n";
			return strReturn;
		}

		private string GetSimpleCharacteristicAbilityCodeInHand(string strText)
		{
			string strPadding = Globals.Padding;
			string strReturn = Utils.PaddingForLevel(strPadding, 1) + "<STATIC_ABILITY" + AddCommaSpace() + " active_zone=\"ZONE_HAND\">\r\n";
			strReturn += GetLocalizedAbilityText();
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<CONTINUOUS_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "RSN_" + strText + "( Object() )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</CONTINUOUS_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 1) + "</STATIC_ABILITY>\r\n";
			return strReturn;
		}

		private string GetAnnihilatorAbilityCode(int nNum)
		{
			string strPadding = Globals.Padding;
			string strReturn = Utils.PaddingForLevel(strPadding, 1) + "<TRIGGERED_ABILITY" + AddCommaSpace() + " badge=\"BADGE_ANNIHILATOR\" filter_zone=\"ZONE_IN_PLAY\">\r\n";
			strReturn += GetLocalizedAbilityText();
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<TRIGGER value=\"ATTACKING\" simple_qualifier=\"self\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<RESOLUTION_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "local defendingPlayer = Object():GetPlayerAttacked()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "local filter = Object():GetFilter()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:Clear()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:NotTargetted()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:SetController( defendingPlayer )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:SetZone( ZONE_IN_PLAY )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:SetHint( HINT_ENEMY, defendingPlayer )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "defendingPlayer:SetTargetCount( " + nNum.ToString() + " )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "for i=0," + (nNum - 1).ToString() + " do\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "defendingPlayer:SetTargetPrompt( i, \"CARD_QUERY_CHOOSE_PERMANENT_TO_SACRIFICE\" )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "end\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "defendingPlayer:ChooseTargets( NO_VALIDATION, EffectDC():Make_Targets(0) )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</RESOLUTION_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<RESOLUTION_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "for i=0," + (nNum - 1).ToString() + " do\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "local target = EffectDC():Get_Targets(0):Get_CardPtr(i)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "if target ~= nil then\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 5) + "local player = target:GetPlayer()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 5) + "target:Sacrifice(player)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "end\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "end\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</RESOLUTION_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 1) + "</TRIGGERED_ABILITY>\r\n";
			return strReturn;
		}

		private string GetBattleCryAbilityCode()
		{
			string strPadding = Globals.Padding;
			string strReturn = Utils.PaddingForLevel(strPadding, 1) + "<TRIGGERED_ABILITY" + AddCommaSpace() + " auto_skip=\"1\" badge=\"BADGE_BATTLE_CRY\" filter_zone=\"ZONE_IN_PLAY\">\r\n";
			strReturn += GetLocalizedAbilityText();
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<TRIGGER value=\"ATTACKING\" simple_qualifier=\"self\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<FILTER>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "return CreaturesInPlay() and (FilteredCard():IsAttacking() ~= 0)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</FILTER>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<CONTINUOUS_ACTION layer=\"7C\">\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "RSN_AddPowerAndToughness( FilteredCard(), 1, 0 )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</CONTINUOUS_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<DURATION simple_duration=\"UntilEOT\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 1) + "</TRIGGERED_ABILITY>\r\n";
			return strReturn;
		}

		private string GetEquipAbilityCode(int nCost)
		{
			string strPadding = Globals.Padding;
			string strReturn = Utils.PaddingForLevel(strPadding, 1) + "<ACTIVATED_ABILITY sorcery_time=\"1\" filter_zone=\"ZONE_IN_PLAY\">\r\n";
			strReturn += GetLocalizedAbilityText();
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<COST type=\"Mana\" cost=\"{" + nCost.ToString() + "}\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<TARGET_DEFINITION id=\"0\">\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "local filter = Object():GetFilter()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:Clear()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:AddCardType( CARD_TYPE_CREATURE )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:SetZone( ZONE_IN_PLAY )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "filter:SetController( EffectController() )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "-- AI 'cannot reequip' exemption\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "local parent = Object():GetParent()\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "if (parent ~= nil) and (EffectController():IsAI() ~= 0) then\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "filter:SetCardInstance(parent)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "filter:AddExtra(FILTER_EXTRA_FLIP_CARD_INSTANCE)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "end\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</TARGET_DEFINITION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<TARGET_DETERMINATION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "return AtLeastOneTargetFromDefinition(0)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</TARGET_DETERMINATION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<PLAY_TIME_ACTION target_choosing=\"1\">\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "EffectController():ChooseTarget( 0, \"CARD_QUERY_CHOOSE_CREATURE_TO_EQUIP\", EffectDC():Make_Targets(0) )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</PLAY_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<RESOLUTION_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "local target_card = EffectDC():Get_Targets(0):Get_CardPtr(0)\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "if (target_card ~= nil and EffectSource() ~= nil) then\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 4) + "EffectSource():Attach( target_card )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "end\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</RESOLUTION_TIME_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<AI_AVAILABILITY type=\"equip\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 1) + "</ACTIVATED_ABILITY>\r\n";
			return strReturn;
		}

		private string GetExaltedAbilityCode()
		{
			string strPadding = Globals.Padding;
			string strReturn = Utils.PaddingForLevel(strPadding, 1) + "<TRIGGERED_ABILITY" + AddCommaSpace() + " badge=\"BADGE_EXALTED\" auto_skip=\"1\" filter_zone=\"ZONE_IN_PLAY\">\r\n";
			strReturn += GetLocalizedAbilityText();
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<TRIGGER value=\"ATTACKING_ALONE\" simple_qualifier=\"objectyoucontrol\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<CONTINUOUS_ACTION layer=\"7C\">\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 3) + "RSN_AddBothPowerAndToughness( TriggerObject(), 1 )\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "</CONTINUOUS_ACTION>\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 2) + "<DURATION simple_duration=\"UntilEOT\" />\r\n";
			strReturn += Utils.PaddingForLevel(strPadding, 1) + "</TRIGGERED_ABILITY>\r\n";
			return strReturn;
		}

		private string GetFlankingAbilityCode()
		{
			string strPadding = Globals.Padding;
			string strReturn = "";
			strReturn += GetLocalizedAbilityText();
			strReturn += "\t\r\n";
			//TODO: Add code.
			return strReturn;
		}
	}
}
