﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DotP2013_Card_Builder
{
	class DotP2013CardInfo : CardInfo
	{
		public DotP2013CardInfo()
			: base()
		{ }

		protected override Ability GetNewAbility()
		{
			return new DotP2013Ability();
		}

		protected override CardType GetNewCardType(string strType)
		{
			return new DotP2013CardType(strType);
		}

		private string CodifyCardName()
		{
			string strName = m_colCardNames["English"].Key.ToUpper();
			strName = strName.Replace(' ', '_');
			strName = strName.Replace(",", "");
			strName = strName.Replace("'", "");
			// Handling for special characters
			strName = strName.Replace("Æ", "AE");
			return strName;
		}

		private string GetLocalizedCardNames()
		{
			string strPadding = Globals.Padding;
			string strDblPadding = strPadding + strPadding;
			string strReturn = strPadding + "<TITLE>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("English") + "\"><![CDATA[" + GetCardName("English") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("French") + "\"><![CDATA[" + GetCardName("French") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Spanish") + "\"><![CDATA[" + GetCardName("Spanish") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("German") + "\"><![CDATA[" + GetCardName("German") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Italian") + "\"><![CDATA[" + GetCardName("Italian") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Japanese") + "\"><![CDATA[" + GetCardName("Japanese") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Korean") + "\"><![CDATA[" + GetCardName("Korean") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Russian") + "\"><![CDATA[" + GetCardName("Russian") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Portuguese") + "\"><![CDATA[" + GetCardName("Portuguese") + "]]></LOCALISED_TEXT>\r\n";
			strReturn += strPadding + "</TITLE>\r\n";
			return strReturn;
		}

		private string GetLocalizedFlavourTextCode()
		{
			if (HasFlavourText())
			{
				string strPadding = Globals.Padding;
				string strDblPadding = strPadding + strPadding;
				string strReturn = strPadding + "<FLAVOURTEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("English") + "\"><![CDATA[" + GetFlavourText("English") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("French") + "\"><![CDATA[" + GetFlavourText("French") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Spanish") + "\"><![CDATA[" + GetFlavourText("Spanish") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("German") + "\"><![CDATA[" + GetFlavourText("German") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Italian") + "\"><![CDATA[" + GetFlavourText("Italian") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Japanese") + "\"><![CDATA[" + GetFlavourText("Japanese") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Korean") + "\"><![CDATA[" + GetFlavourText("Korean") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Russian") + "\"><![CDATA[" + GetFlavourText("Russian") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strDblPadding + "<LOCALISED_TEXT LanguageCode=\"" + Utils.ConvertLanguageToLangCode("Portuguese") + "\"><![CDATA[" + GetFlavourText("Portuguese") + "]]></LOCALISED_TEXT>\r\n";
				strReturn += strPadding + "</FLAVOURTEXT>\r\n";
				return strReturn;
			}
			else
				return "";
		}

		private string ConvertRarityToCode()
		{
			string strReturn = "";
			switch (RarityEnum)
			{
				case CardRarity.Land:
					strReturn = "L";
					break;
				case CardRarity.Common:
					strReturn = "C";
					break;
				case CardRarity.Uncommon:
					strReturn = "U";
					break;
				case CardRarity.Rare:
					strReturn = "R";
					break;
				case CardRarity.Mythic:
					strReturn = "M";
					break;
				case CardRarity.Special:
					// We'll use Promo for special rarity.
					strReturn = "P";
					break;
				default:
					// We don't know what the game will actually accept for Special/Promo rarities so we'll use blank.
					strReturn = "";
					break;
			}
			return strReturn;
		}

		private string ConvertExpansionToCode()
		{
			string strReturn = "";
			if (Expansion.Equals("Limited Edition Alpha", StringComparison.OrdinalIgnoreCase))
				strReturn = "LEA";
			else if (Expansion.Equals("Limited Edition Beta", StringComparison.OrdinalIgnoreCase))
				strReturn = "LEB";
			else if (Expansion.Equals("Unlimited Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "2ED";
			else if (Expansion.Equals("Revised Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "3ED";
			else if (Expansion.Equals("Fourth Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "4ED";
			else if (Expansion.Equals("Fifth Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "5ED";
			else if (Expansion.Equals("Classic Sixth Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "6ED";
			else if (Expansion.Equals("Seventh Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "7ED";
			else if (Expansion.Equals("Eighth Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "8ED";
			else if (Expansion.Equals("Ninth Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "9ED";
			else if (Expansion.Equals("Tenth Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "10E";
			else if (Expansion.Equals("Magic 2010", StringComparison.OrdinalIgnoreCase))
				strReturn = "M10";
			else if (Expansion.Equals("Magic 2011", StringComparison.OrdinalIgnoreCase))
				strReturn = "M11";
			else if (Expansion.Equals("Magic 2012", StringComparison.OrdinalIgnoreCase))
				strReturn = "M12";
			else if (Expansion.Equals("Magic 2013", StringComparison.OrdinalIgnoreCase))
				strReturn = "M13";
			else if (Expansion.Equals("Alara Reborn", StringComparison.OrdinalIgnoreCase))
				strReturn = "ARB";
			else if (Expansion.Equals("Alliances", StringComparison.OrdinalIgnoreCase))
				strReturn = "ALL";
			else if (Expansion.Equals("Anthologies", StringComparison.OrdinalIgnoreCase))
				strReturn = "ATH";
			else if (Expansion.Equals("Antiquities", StringComparison.OrdinalIgnoreCase))
				strReturn = "ATQ";
			else if (Expansion.Equals("Apocalypse", StringComparison.OrdinalIgnoreCase))
				strReturn = "APC";
			else if (Expansion.Equals("Arabian Nights", StringComparison.OrdinalIgnoreCase))
				strReturn = "ARN";
			else if (Expansion.Equals("Archenemy", StringComparison.OrdinalIgnoreCase))
				strReturn = "ARC";
			else if (Expansion.Equals("Astral", StringComparison.OrdinalIgnoreCase))
				strReturn = "AST";
			else if (Expansion.Equals("Avacyn Restored", StringComparison.OrdinalIgnoreCase))
				strReturn = "AVR";
			else if (Expansion.Equals("Battle Royale Box Set", StringComparison.OrdinalIgnoreCase))
				strReturn = "BRB";
			else if (Expansion.Equals("Beatdown Box Set", StringComparison.OrdinalIgnoreCase))
				strReturn = "BTD";
			else if (Expansion.Equals("Betrayers of Kamigawa", StringComparison.OrdinalIgnoreCase))
				strReturn = "BOK";
			else if (Expansion.Equals("Champions of Kamigawa", StringComparison.OrdinalIgnoreCase))
				strReturn = "CHK";
			else if (Expansion.Equals("Chronicles", StringComparison.OrdinalIgnoreCase))
				strReturn = "CHR";
			else if (Expansion.Equals("Coldsnap", StringComparison.OrdinalIgnoreCase))
				strReturn = "CSP";
			else if (Expansion.Equals("Conflux", StringComparison.OrdinalIgnoreCase))
				strReturn = "CFX";
			else if (Expansion.Equals("Dark Ascension", StringComparison.OrdinalIgnoreCase))
				strReturn = "DKA";
			else if (Expansion.Equals("Darksteel", StringComparison.OrdinalIgnoreCase))
				strReturn = "DST";
			else if (Expansion.Equals("Dissension", StringComparison.OrdinalIgnoreCase))
				strReturn = "DIS";
			else if (Expansion.Equals("Dreamcast (Sega)", StringComparison.OrdinalIgnoreCase))
				strReturn = "SDC";
			else if (Expansion.Equals("Duel Decks: Ajani vs. Nicol Bolas", StringComparison.OrdinalIgnoreCase))
				strReturn = "AVN"; // I don't know the real one so I'm using this.
			else if (Expansion.Equals("Duel Decks: Divine vs. Demonic", StringComparison.OrdinalIgnoreCase))
				strReturn = "DVD";
			else if (Expansion.Equals("Duel Decks: Elspeth vs. Tezzeret", StringComparison.OrdinalIgnoreCase))
				strReturn = "DDF";
			else if (Expansion.Equals("Duel Decks: Elves vs. Goblins", StringComparison.OrdinalIgnoreCase))
				strReturn = "EVG";
			else if (Expansion.Equals("Duel Decks: Garruk vs. Liliana", StringComparison.OrdinalIgnoreCase))
				strReturn = "GVL";
			else if (Expansion.Equals("Duel Decks: Jace vs. Chandra", StringComparison.OrdinalIgnoreCase))
				strReturn = "JVC";
			else if (Expansion.Equals("Duel Decks: Knights vs. Dragons", StringComparison.OrdinalIgnoreCase))
				strReturn = "KVD"; // I don't know the real one so I'm using this.
			else if (Expansion.Equals("Duel Decks: Phyrexia vs. the Coalition", StringComparison.OrdinalIgnoreCase))
				strReturn = "PVC"; // I don't know the real one so I'm using this.
			else if (Expansion.Equals("Duel Decks: Venser vs. Koth", StringComparison.OrdinalIgnoreCase))
				strReturn = "VVK";
			else if (Expansion.Equals("Eventide", StringComparison.OrdinalIgnoreCase))
				strReturn = "EVE";
			else if (Expansion.Equals("Exodus", StringComparison.OrdinalIgnoreCase))
				strReturn = "EXO";
			else if (Expansion.Equals("Fallen Empires", StringComparison.OrdinalIgnoreCase))
				strReturn = "FEM";
			else if (Expansion.Equals("Fifth Dawn", StringComparison.OrdinalIgnoreCase))
				strReturn = "5DN";
			else if (Expansion.Equals("From the Vault: Dragons", StringComparison.OrdinalIgnoreCase))
				strReturn = "FVD";
			else if (Expansion.Equals("From the Vault: Exiled", StringComparison.OrdinalIgnoreCase))
				strReturn = "FVE";
			else if (Expansion.Equals("From the Vault: Legends", StringComparison.OrdinalIgnoreCase))
				strReturn = "FVL";
			else if (Expansion.Equals("From the Vault: Relics", StringComparison.OrdinalIgnoreCase))
				strReturn = "FVR";
			else if (Expansion.Equals("Future Sight", StringComparison.OrdinalIgnoreCase))
				strReturn = "FUT";
			else if (Expansion.Equals("Guildpact", StringComparison.OrdinalIgnoreCase))
				strReturn = "GPT";
			else if (Expansion.Equals("Homelands", StringComparison.OrdinalIgnoreCase))
				strReturn = "HML";
			else if (Expansion.Equals("Ice Age", StringComparison.OrdinalIgnoreCase))
				strReturn = "ICE";
			else if (Expansion.Equals("Innistrad", StringComparison.OrdinalIgnoreCase))
				strReturn = "ISD";
			else if (Expansion.Equals("Invasion", StringComparison.OrdinalIgnoreCase))
				strReturn = "INV";
			else if (Expansion.Equals("Judgment", StringComparison.OrdinalIgnoreCase))
				strReturn = "JUD";
			else if (Expansion.Equals("Legends", StringComparison.OrdinalIgnoreCase))
				strReturn = "LEG";
			else if (Expansion.Equals("Legions", StringComparison.OrdinalIgnoreCase))
				strReturn = "LGN";
			else if (Expansion.Equals("Lorwyn", StringComparison.OrdinalIgnoreCase))
				strReturn = "LRW";
			else if (Expansion.Equals("Magic: The Gathering-Commander", StringComparison.OrdinalIgnoreCase))
				strReturn = "COM";
			else if (Expansion.Equals("Masters Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "MED";
			else if (Expansion.Equals("Masters Edition II", StringComparison.OrdinalIgnoreCase))
				strReturn = "ME2";
			else if (Expansion.Equals("Masters Edition III", StringComparison.OrdinalIgnoreCase))
				strReturn = "ME3";
			else if (Expansion.Equals("Masters Edition IV", StringComparison.OrdinalIgnoreCase))
				strReturn = "ME4";
			else if (Expansion.Equals("Mercadian Masques", StringComparison.OrdinalIgnoreCase))
				strReturn = "MMQ";
			else if (Expansion.Equals("Mirage", StringComparison.OrdinalIgnoreCase))
				strReturn = "MIR";
			else if (Expansion.Equals("Mirrodin", StringComparison.OrdinalIgnoreCase))
				strReturn = "MRD";
			else if (Expansion.Equals("Mirrodin Besieged", StringComparison.OrdinalIgnoreCase))
				strReturn = "MBS";
			else if (Expansion.Equals("Morningtide", StringComparison.OrdinalIgnoreCase))
				strReturn = "MOR";
			else if (Expansion.Equals("Multiverse", StringComparison.OrdinalIgnoreCase))
				strReturn = "MUV";
			else if (Expansion.Equals("Nemesis", StringComparison.OrdinalIgnoreCase))
				strReturn = "NMS";
			else if (Expansion.Equals("New Phyrexia", StringComparison.OrdinalIgnoreCase))
				strReturn = "NPH";
			else if (Expansion.Equals("Odyssey", StringComparison.OrdinalIgnoreCase))
				strReturn = "ODY";
			else if (Expansion.Equals("Onslaught", StringComparison.OrdinalIgnoreCase))
				strReturn = "ONS";
			else if (Expansion.Equals("Planar Chaos", StringComparison.OrdinalIgnoreCase))
				strReturn = "PLC";
			else if (Expansion.Equals("Planechase", StringComparison.OrdinalIgnoreCase))
				strReturn = "PCH";
			else if (Expansion.Equals("Planechase 2012 Edition", StringComparison.OrdinalIgnoreCase))
				strReturn = "P12"; // I don't know the real one so I'm using this.
			else if (Expansion.Equals("Planeshift", StringComparison.OrdinalIgnoreCase))
				strReturn = "PLS";
			else if (Expansion.Equals("Portal", StringComparison.OrdinalIgnoreCase))
				strReturn = "POR";
			else if (Expansion.Equals("Portal Second Age", StringComparison.OrdinalIgnoreCase))
				strReturn = "P02";
			else if (Expansion.Equals("Portal Three Kingdoms", StringComparison.OrdinalIgnoreCase))
				strReturn = "PTK";
			else if (Expansion.Equals("Premium Deck Series: Fire and Lightning", StringComparison.OrdinalIgnoreCase))
				strReturn = "PD2";
			else if (Expansion.Equals("Premium Deck Series: Graveborn", StringComparison.OrdinalIgnoreCase))
				strReturn = "PD3";
			else if (Expansion.Equals("Premium Deck Series: Slivers", StringComparison.OrdinalIgnoreCase))
				strReturn = "SLI";
			else if (Expansion.Equals("Promo set for Gatherer", StringComparison.OrdinalIgnoreCase))
				strReturn = "PRO";
			else if (Expansion.Equals("Prophecy", StringComparison.OrdinalIgnoreCase))
				strReturn = "PCY";
			else if (Expansion.Equals("Ravnica: City of Guilds", StringComparison.OrdinalIgnoreCase))
				strReturn = "RAV";
			else if (Expansion.Equals("Rise of the Eldrazi", StringComparison.OrdinalIgnoreCase))
				strReturn = "ROE";
			else if (Expansion.Equals("Saviors of Kamigawa", StringComparison.OrdinalIgnoreCase))
				strReturn = "SOK";
			else if (Expansion.Equals("Scars of Mirrodin", StringComparison.OrdinalIgnoreCase))
				strReturn = "SOM";
			else if (Expansion.Equals("Scourge", StringComparison.OrdinalIgnoreCase))
				strReturn = "SCG";
			else if (Expansion.Equals("Shadowmoor", StringComparison.OrdinalIgnoreCase))
				strReturn = "SHM";
			else if (Expansion.Equals("Shards of Alara", StringComparison.OrdinalIgnoreCase))
				strReturn = "ALA";
			else if (Expansion.Equals("Starter 1999", StringComparison.OrdinalIgnoreCase))
				strReturn = "S99";
			else if (Expansion.Equals("Starter 2000", StringComparison.OrdinalIgnoreCase))
				strReturn = "S00";
			else if (Expansion.Equals("Stronghold", StringComparison.OrdinalIgnoreCase))
				strReturn = "STH";
			else if (Expansion.Equals("Tempest", StringComparison.OrdinalIgnoreCase))
				strReturn = "TMP";
			else if (Expansion.Equals("The Dark", StringComparison.OrdinalIgnoreCase))
				strReturn = "DRK";
			else if (Expansion.Equals("Time Spiral", StringComparison.OrdinalIgnoreCase))
				strReturn = "TSP";
			else if (Expansion.Equals("Time Spiral \"Timeshifted\"", StringComparison.OrdinalIgnoreCase))
				strReturn = "TSB";
			else if (Expansion.Equals("Torment", StringComparison.OrdinalIgnoreCase))
				strReturn = "TOR";
			else if (Expansion.Equals("Unglued", StringComparison.OrdinalIgnoreCase))
				strReturn = "UGL";
			else if (Expansion.Equals("Unhinged", StringComparison.OrdinalIgnoreCase))
				strReturn = "UNH";
			else if (Expansion.Equals("Urza's Destiny", StringComparison.OrdinalIgnoreCase))
				strReturn = "UDS";
			else if (Expansion.Equals("Urza's Legacy", StringComparison.OrdinalIgnoreCase))
				strReturn = "ULG";
			else if (Expansion.Equals("Urza's Saga", StringComparison.OrdinalIgnoreCase))
				strReturn = "USG";
			else if (Expansion.Equals("Vanguard", StringComparison.OrdinalIgnoreCase))
				strReturn = "VGD";
			else if (Expansion.Equals("Visions", StringComparison.OrdinalIgnoreCase))
				strReturn = "VIS";
			else if (Expansion.Equals("Weatherlight", StringComparison.OrdinalIgnoreCase))
				strReturn = "WTH";
			else if (Expansion.Equals("Worldwake", StringComparison.OrdinalIgnoreCase))
				strReturn = "WWK";
			else if (Expansion.Equals("Zendikar", StringComparison.OrdinalIgnoreCase))
				strReturn = "ZEN";
			return strReturn;
		}

		public override string GetCardCode()
		{
			string strPadding = Globals.Padding;
			string strCardName = CodifyCardName();
			string strArtist = Utils.PrepareXmlString(Artist);
			string strReturn = "<?xml version='1.0'?>\r\n";
			strReturn += "<CARD_V2>\r\n";
			strReturn += strPadding + "<FILENAME text=\"" + strCardName + "_" + MultiverseId.ToString() + "\" />\r\n";
			strReturn += strPadding + "<CARDNAME text=\"" + strCardName + "\" />\r\n";
			strReturn += GetLocalizedCardNames();
			strReturn += strPadding + "<MULTIVERSEID value=\"" + MultiverseId.ToString() + "\" />\r\n";
			strReturn += strPadding + "<ARTID value=\"RSN" + MultiverseId.ToString() + "\" />\r\n";
			strReturn += strPadding + "<ARTIST name=\"" + strArtist + "\" />\r\n";
			strReturn += strPadding + "<CASTING_COST cost=\"" + CastingCost + "\" />\r\n";
			strReturn += GetLocalizedFlavourTextCode();
			strReturn += CardType.GetCardText();
			strReturn += strPadding + "<EXPANSION value=\"" + ConvertExpansionToCode() + "\" />\r\n";
			strReturn += strPadding + "<RARITY metaname=\"" + ConvertRarityToCode() + "\" />\r\n";
			if (Power.Length > 0)
				strReturn += strPadding + "<POWER value=\"" + Power + "\" />\r\n";
			if (Toughness.Length > 0)
				strReturn += strPadding + "<TOUGHNESS value=\"" + Toughness + "\" />\r\n";
			foreach (Ability abAbility in Abilities)
				strReturn += abAbility.GetCardAbilityCode();
			strReturn += "</CARD_V2>";
			return strReturn;
		}
	}
}
