﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace DotP2013_Card_Builder
{
	enum ErrorPriority
	{
		Critical = 4,
		High = 3,
		Medium = 2,
		Low = 1,
		Zero = 0
	}

	class Globals
	{
		// Public Globals (including defaults)
		public static string Padding = "\t";
		public static bool IncludeBOM = true;
		public static bool LogAllErrors = false;

		// Private Globals (typically log objects and the like)
		private static XmlDocument Settings = new XmlDocument();
		private static MessageLog ErrorLog = new MessageLog("Errors.log");
		private static MessageLog RunLog = new MessageLog("Messages.log");

		// Global functions (This is for commonly used functions that are best put here)
		public static void LoadSettings()
		{
			ReportMessage("Globals", "Opening Settings File ...");
			Settings = new XmlDocument();
			try
			{
				Settings.Load("Settings.xml");

				// Here is where we recover the global settings
				Padding = GetSetting("Padding", Padding);
				IncludeBOM = GetSetting("IncludeBOM", IncludeBOM);
				LogAllErrors = GetSetting("LogAllErrors", LogAllErrors);
				ReportMessage("Globals", "Settings Loaded, Globals Set.");
			}
			catch (Exception e)
			{
				ReportError(e, ErrorPriority.Low);
				ReportMessage("Globals", "Unable to read in Settings File.");
			}
		}

		public static void ReportError(Exception eError, ErrorPriority epPriority, string strOptionalInfo = "")
		{
			string strExtraInfo = "";
			if (strOptionalInfo.Length > 0)
				strExtraInfo = "\r\nExtra Information:\r\n" + strOptionalInfo;
			if ((epPriority != ErrorPriority.Zero) || (LogAllErrors))
				ErrorLog.WriteMessage(epPriority.ToString() + ": " + eError.Source + " - " + eError.Message + ":\r\n" + eError.StackTrace + strExtraInfo);
		}

		public static void ReportMessage(string strSection, string strMessage)
		{
			RunLog.WriteMessage(strSection + ": " + strMessage);
		}

		public static string GetSetting(string strSettingName, string strDefault)
		{
			ReportMessage("Globals", "Retrieving Setting \"" + strSettingName + "\" ...");
			string strReturn = strDefault;
			XmlNodeList xnlSettings = Settings.GetElementsByTagName(strSettingName);
			if (xnlSettings.Count > 0)
			{
				XmlNode xnSetting = xnlSettings[0];
				if (xnSetting.Attributes["value"] != null)
				{
					strReturn = xnSetting.Attributes["value"].Value;
					ReportMessage("Globals", "Setting Retrieved = \"" + strReturn.ToString() + "\"");
				}
				else
					ReportMessage("Globals", "Could not retrieve setting, using default = " + strReturn.ToString());
			}
			return strReturn;
		}

		public static bool GetSetting(string strSettingName, bool bDefault)
		{
			ReportMessage("Globals", "Retrieving Setting \"" + strSettingName + "\" ...");
			bool bReturn = bDefault;
			XmlNodeList xnlSettings = Settings.GetElementsByTagName(strSettingName);
			if (xnlSettings.Count > 0)
			{
				XmlNode xnSetting = xnlSettings[0];
				if (xnSetting.Attributes["value"] != null)
				{
					try
					{
						bReturn = Boolean.Parse(xnSetting.Attributes["value"].Value);
						ReportMessage("Globals", "Setting Retrieved = " + bReturn.ToString());
					}
					catch (Exception e)
					{
						// We don't particularly care if it fails to parse the value for the setting because we'll just use the default.
						ReportError(e, ErrorPriority.Zero);
						ReportMessage("Globals", "Could not retrieve setting, using default = " + bReturn.ToString());
					}
				}
			}
			return bReturn;
		}

		public static int GetSetting(string strSettingName, int nDefault)
		{
			ReportMessage("Globals", "Retrieving Setting \"" + strSettingName + "\" ...");
			int nReturn = nDefault;
			XmlNodeList xnlSettings = Settings.GetElementsByTagName(strSettingName);
			if (xnlSettings.Count > 0)
			{
				XmlNode xnSetting = xnlSettings[0];
				if (xnSetting.Attributes["value"] != null)
				{
					try
					{
						nReturn = Int32.Parse(xnSetting.Attributes["value"].Value);
						ReportMessage("Globals", "Setting Retrieved = " + nReturn.ToString());
					}
					catch (Exception e)
					{
						// We don't particularly care if it fails to parse the value for the setting because we'll just use the default.
						ReportError(e, ErrorPriority.Zero);
						ReportMessage("Globals", "Could not retrieve setting, using default = " + nReturn.ToString());
					}
				}
			}
			return nReturn;
		}
	}
}
